#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['o2g',
 'o2g.gtfs',
 'o2g.osm',
 'o2g.osm.builders',
 'o2g.osm.handlers',
 'o2g.tests']

package_data = \
{'': ['*']}

install_requires = \
['osmium']

extras_require = \
{'test': ['pytest'], 'web': ['bottle']}

entry_points = \
{'console_scripts': ['o2g = o2g.cli:cli']}

setup(name='o2g',
      version='0.5.1',
      description='A simple tool to extract GTFS feed from OpenStreetMap.',
      author='Mehdi Sadeghi',
      author_email='mehdi@mehdix.rog',
      url='https://github.com/hiposfer/o2g',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.5',
     )
